"""
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        renamer.py
Date:           11/04/14
Description:    This script will rename the selected nodes.
Installation:   Place this file in your maya scripts directory
                (e.g. ..\My Documents\maya\scripts) then run the following
                python commands:
                    import renamer
                    renamer.main()
"""
import maya.cmds as cmds
import re


class Renamer(object):
    def __init__(self):
        self.remove_first_integer = None
        self.remove_last_integer = None
        self.find_text = None
        self.replace_text = None
        self.match_case_checkbox = None
        self.first_only_checkbox = None
        self.prefix_text = None
        self.base_text = None
        self.suffix_text = None
        self.number_padding_radio = None
        self.alpha_padding_radio = None
        self.no_padding_radio = None
        self.number_padding_integer = None
        self.no_case_radio = None
        self.uppercase_radio = None
        self.lowercase_radio = None
        self.shorten_name_checkbox = None
        self.shorten_name_integer = None
        self.reset_fields_checkbox = None
        self.preview_checkbox = None
        self.status_label = None

    @staticmethod
    def get_base_name(long_name):
        """Returns the root name of 'long_name' """
        root_name = (long_name.split(":"))[-1]
        root_name = (root_name.split("|"))[-1]

        return root_name

    @staticmethod
    def get_remove(name, start_index, end_index):
        """Performs a remove first/last X number and returns the result"""
        if start_index > 0:
            name = name[start_index:]
        if end_index > 0:
            name = name[:-end_index]

        return name

    @staticmethod
    def get_find_replace(name, find_text, replace_text, match_case, first_only):
        """Performs a find and replace and returns the result"""
        count = 0
        
        if first_only:
            count = 1
        
        if find_text:
            if match_case:
                if count == 0:
                    # .replace takes -1 for 'all occurrences'
                    count = -1
                name = name.replace(find_text, replace_text, count)
            else:
                # CRITICAL! You must escape the special characters
                special_chars = '.^$*+?{}[]|()'
                find_text = find_text.replace('\\', '\\\\')
                for char in special_chars:
                    find_text = find_text.replace(char, '\\' + char)

                name = re.sub(find_text, replace_text, name, count, re.IGNORECASE)

        return name

    @staticmethod
    def get_edit(name, user_prefix, user_base, user_suffix):
        """Performs a name edit and returns the result"""
        if user_base:
            name = user_base

        name = user_prefix + name
        name += user_suffix

        return name

    @staticmethod
    def get_numeric_padding(counter, user_padding):
        """Returns a numeric padding string based on 'counter' """
        result = str(counter)
        result = result.zfill(user_padding)

        return result

    @staticmethod
    def get_alpha_padding(counter):
        """Returns an alphabetic padding string based on 'counter' """
        total_padding = int(counter / 27)
        letter = chr(counter - (26 * total_padding) + 64)
        result = letter
        for i in range(0, total_padding, 1):
            result += letter

        return result

    def get_padding(self, name, number_padding, padding, alpha_padding, counter):
        """Returns 'name' with a padding value (numeric or alphabetic) if
        required"""
        if number_padding:
            name += self.get_numeric_padding(counter, padding)
        if alpha_padding:
            name += self.get_alpha_padding(counter)

        return name

    @staticmethod
    def get_casing(name, to_upper, to_lower):
        """Performs a casing operation and returns the result"""
        if to_upper:
            name = name.upper()
        if to_lower:
            name = name.lower()

        return name

    @staticmethod
    def get_shorten(name, shorten_name, max_length):
        """Performs a trim operation on 'name' and returns the result"""
        if shorten_name:
            name = name[:max_length]

        return name

    def reset_ui(self):
        """Resets the UI controls to their default values"""
        cmds.intField(self.remove_first_integer, edit=True, value=0)
        cmds.intField(self.remove_last_integer, edit=True, value=0)
        cmds.textFieldGrp(self.find_text, edit=True, text="")
        cmds.textFieldGrp(self.replace_text, edit=True, text="")
        cmds.checkBox(self.match_case_checkbox, edit=True, value=True)
        cmds.checkBox(self.first_only_checkbox, edit=True, value=False)
        cmds.textFieldGrp(self.prefix_text, edit=True, text="")
        cmds.textFieldGrp(self.base_text, edit=True, text="")
        cmds.textFieldGrp(self.suffix_text, edit=True, text="")
        cmds.radioButton(self.no_padding_radio, edit=True, select=True)
        cmds.intField(self.number_padding_integer, edit=True, value=3)
        cmds.radioButton(self.no_case_radio, edit=True, select=True)
        cmds.checkBox(self.shorten_name_checkbox, edit=True, value=False)
        cmds.intField(self.shorten_name_integer, edit=True, value=60)

    def rename_button_pressed(self, *args):
        """Rename button event handler. Renames the selected nodes"""
        selected_nodes = cmds.ls(selection=True, long=True)
        counter = len(selected_nodes)
        preview_mode = cmds.checkBox(self.preview_checkbox, query=True, value=True)
        error_counter = 0

        if not selected_nodes:
            cmds.text(self.status_label, edit=True, label='ERROR: Nothing selected')
            return False

        selected_nodes.sort()

        if preview_mode:
            print('***   BEGIN RENAME PREVIEW   ***')

        for current in reversed(selected_nodes):
            name = self.get_base_name(current)
            name = self.get_remove(name,
                                   cmds.intField(self.remove_first_integer, query=True, value=True),
                                   cmds.intField(self.remove_last_integer, query=True, value=True))
            name = self.get_find_replace(name,
                                         cmds.textFieldGrp(self.find_text, query=True, text=True),
                                         cmds.textFieldGrp(self.replace_text, query=True, text=True),
                                         cmds.checkBox(self.match_case_checkbox, query=True, value=True),
                                         cmds.checkBox(self.first_only_checkbox, query=True, value=True))
            name = self.get_edit(name,
                                 cmds.textFieldGrp(self.prefix_text, query=True, text=True),
                                 cmds.textFieldGrp(self.base_text, query=True, text=True),
                                 cmds.textFieldGrp(self.suffix_text, query=True, text=True))
            name = self.get_padding(name,
                                    cmds.radioButton(self.number_padding_radio, query=True, select=True),
                                    cmds.intField(self.number_padding_integer,  query=True, value=True),
                                    cmds.radioButton(self.alpha_padding_radio, query=True, select=True),
                                    counter)
            # CRITICAL! Decrement counter now!
            counter -= 1

            name = self.get_casing(name,
                                   cmds.radioButton(self.uppercase_radio, query=True, select=True),
                                   cmds.radioButton(self.lowercase_radio, query=True, select=True))
            name = self.get_shorten(name,
                                    cmds.checkBox(self.shorten_name_checkbox, query=True, value=True),
                                    cmds.intField(self.shorten_name_integer, query=True, value=True))
            
            if preview_mode:
                print(current + ' -> ' + name)
            else:
                if not name:
                    print('ERROR: New name for ' + current + ' is null. Cancelling rename.')
                    error_counter += 1
                    continue

                final_name = cmds.rename(current, name)
                final_name = self.get_base_name(final_name)
                if final_name != name:
                    print('ERROR: Rename for object ' + current + ' failed.')
                    print('   Attempted: ' + name)
                    print('   Result: ' + final_name)
                    error_counter += 1

        if preview_mode:
            print('***   END RENAME PREVIEW   ***')
            cmds.text(self.status_label, edit=True, label='INFO: Check script editor for preview')
        else:
            if cmds.checkBox(self.reset_fields_checkbox, query=True, value=True):
                self.reset_ui()

            if error_counter == 0:
                cmds.text(self.status_label, edit=True,
                          label='INFO: Rename complete (' + str(len(selected_nodes)) + ' nodes)')
            else:
                cmds.text(self.status_label, edit=True, label='WARNING: Check script editor')

    def setup_ui(self):
        """Creates the UI"""
        if cmds.window('renamerWindow', exists=True):
            cmds.deleteUI('renamerWindow')

        rename_window = cmds.window('renamerWindow', title='Renamer',
                                    width=220, height=570,
                                    minimizeButton=False, maximizeButton=False,
                                    sizeable=False, resizeToFitChildren=True)
        cmds.columnLayout(width=220)

        # Remove Controls
        cmds.frameLayout(width=220, label='1. Remove', collapsable=False, collapse=False,
                         marginHeight=5, marginWidth=5)
        cmds.rowLayout(width=220, numberOfColumns=4, columnWidth4=[30, 50, 30, 50],
                       columnAlign4=['left', 'left', 'left', 'left'])
        cmds.text(label='First:')
        self.remove_first_integer = cmds.intField(width=40, minValue=0,
                                                  maxValue=99, value=0, step=1)
        cmds.text(label='Last:')
        self.remove_last_integer = cmds.intField(width=40, minValue=0, maxValue=99,
                                                 value=0, step=1)
        cmds.setParent('..')  # end rowLayout
        cmds.setParent('..')  # end frameLayout

        # Find/Replace Controls
        cmds.frameLayout(width=220, label='2. Find and Replace', collapsable=False,
                         collapse=False, marginHeight=5, marginWidth=5)
        cmds.columnLayout(width=220)
        self.find_text = cmds.textFieldGrp(label='Find:', columnAlign=[1, 'left'],
                                           columnWidth2=[50, 155])
        self.replace_text = cmds.textFieldGrp(label='Replace:', columnAlign=[1, 'left'],
                                              columnWidth2=[50, 155])
        self.match_case_checkbox = cmds.checkBox(label='Match Case', value=True)
        self.first_only_checkbox = cmds.checkBox(label='First Occurrence Only')
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end frameLayout

        # Edit Controls
        cmds.frameLayout(width=220, label='3. Edit Name', collapsable=False,
                         collapse=False, marginHeight=5, marginWidth=5)
        cmds.columnLayout(width=220)
        self.prefix_text = cmds.textFieldGrp(label='Prefix:', columnAlign=[1, 'left'],
                                             columnWidth2=[50, 155])
        self.base_text = cmds.textFieldGrp(label='Base:', columnAlign=[1, 'left'],
                                           columnWidth2=[50, 155])
        self.suffix_text = cmds.textFieldGrp(label='Suffix:', columnAlign=[1,'left'],
                                             columnWidth2=[50, 155])
        cmds.setParent('..')  # end columnLayout
        cmds.rowColumnLayout(width=220, numberOfColumns=3,
                             columnWidth=[(1, 125), (2, 55), (3, 25)])
        cmds.radioCollection()
        self.number_padding_radio = cmds.radioButton(label='Numbered')
        cmds.text(label='Padding:')
        self.number_padding_integer = cmds.intField(width=20, minValue=1, maxValue=9,
                                                    step=1, value=3)
        self.alpha_padding_radio = cmds.radioButton(label='Lettered')
        cmds.text(label='')
        cmds.text(label='')
        self.no_padding_radio = cmds.radioButton(label='None', select=True)
        cmds.setParent('..')  # end rowColumnLayout
        cmds.setParent('..')  # end frameLayout

        # Misc Controls
        cmds.frameLayout(width=220, label='4. Misc', collapsable=False,
                         collapse=False, marginHeight=5, marginWidth=5)
        cmds.rowColumnLayout(width=220, numberOfColumns=2,
                             columnWidth=[(1, 50), (2, 150)],
                             columnAlign=[(1, 'left'), (2, 'left')])
        cmds.text(label='Casing:')
        cmds.radioCollection()
        self.no_case_radio = cmds.radioButton(label='No Change', select=True)
        cmds.text(label='')
        self.uppercase_radio = cmds.radioButton(label='UPPERCASE')
        cmds.text(label='')
        self.lowercase_radio = cmds.radioButton(label='lowercase')
        cmds.setParent('..')  # end rowColumnLayout
        cmds.rowColumnLayout(width=220, numberOfColumns=2,
                             columnWidth=[(1, 175), (2, 30)],
                             columnAlign=[(1, 'left'), (2, 'left')])
        self.shorten_name_checkbox = cmds.checkBox(label='Shorten Names Longer Than:')
        self.shorten_name_integer = cmds.intField(width=30, minValue=10,
                                                  maxValue=999, value=60, step=1)
        cmds.setParent('..')  # end rowColumnLayout
        cmds.setParent('..')  # end frameLayout

        # Options
        cmds.frameLayout(width=220, label='Options', collapsable=False,
                         collapse=False, marginHeight=5, marginWidth=5)
        cmds.columnLayout(width=200)
        self.reset_fields_checkbox = cmds.checkBox(label='Reset All Fields After Rename',
                                                   value=True)
        self.preview_checkbox = cmds.checkBox(label='Preview New Names (Print Only)')
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end frameLayout

        cmds.columnLayout(width=220, rowSpacing=5, columnAttach=['left', 5])
        cmds.text(label='', height=5)
        cmds.button(width=205, label='Rename Selected Objects Now',
                    command=self.rename_button_pressed)
        self.status_label = cmds.text(label='INFO: Standing by...')
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end columnLayout

        cmds.showWindow(rename_window)


def main():
    """Main entry point. Opens the UI"""
    renamer = Renamer()
    renamer.setup_ui()


if __name__ == "__main__":
    main()
